<?php

namespace WebsiteBundle\Controller\Site;

use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\Security\Core\Exception\AccessDeniedException;
use WebsiteBundle\Controller\Base\BaseUserController;
use WebsiteBundle\Entity\PsychEducationPanel;
use WebsiteBundle\Entity\Test;
use WebsiteBundle\Entity\TestSession;
use WebsiteBundle\Form\Site\ReminderType;
use WebsiteBundle\Form\Site\SkipTutorialForm;

class SessionController extends BaseUserController
{
  public function psychoEducationAction(Request $request)
  {
    $user = $this->getUser();

    //have they agreed to the terms and conditions? if not, by going to this page they automatically have
    if (!$user->isAgreesToTermsAndConditions()) {
      $now = new \DateTime('now', new \DateTimeZone('UTC'));
      $user
        ->setAgreesToTermsAndConditions(true)
        ->setConsented(true)
        ->setConsentedOn($now)
      ;
      $entityManager = $this->get('doctrine.orm.entity_manager');
      $entityManager->persist($user);
      $entityManager->flush();
    }

    if ($user->hasCompletedPsychEducation()) {
      return $this->redirectToRoute('session_survey');
    }

    try {
      $experiment = $this->getExperiment();
    } catch (AccessDeniedException $e) {
      return $this->handleAccessDeniedException($request, 'user_login');
    }

    //Get the session
    $entityRepo = $this->get('doctrine.orm.entity_manager')->getRepository(PsychEducationPanel::class);
    $psychoEduPanels = $entityRepo->getPsychEducationPanels($experiment);

    $templateData = [
      'panels' => $psychoEduPanels,
      'headerTitle' => 'PREVEND'
    ];

    return $this->render('@Website/Session/psychoEducation.html.twig', $templateData);
  }

  public function surveyAction(Request $request)
  {
    try {
      $templateData = $this->getBasicData();
    } catch (AccessDeniedException $e) {
      return $this->handleAccessDeniedException($request, 'user_login');
    }

    //If they're finished, send em back
    if ($this->getUser()->hasCompletedAllSessions()) {
      return $this->redirectToRoute('user_home');
    }

    $templateData['headerTitle'] = 'ENQUETE';

    return $this->render('@Website/Session/survey.html.twig', $templateData);
  }

  public function tutorialAction(Request $request)
  {
    try {
      $templateData = $this->getBasicData();
    } catch (AccessDeniedException $e) {
      return $this->handleAccessDeniedException($request, 'user_login');
    }

    $testSession = $templateData['testSession'];
    $experiment = $templateData['experiment'];

    //Check if we need to skip the tutorial, don't allow skipping for the last experiment
    $user = $this->getUser();
    if ($user->isSkipTutorial() && $testSession['number'] != $experiment->getTotalSessions()) {
      return $this->redirectToRoute('session_test');
    }

    //Create the form
    $form = $this->createForm(SkipTutorialForm::class, $user);

    //Handle the form
    $form->handleRequest($request);
    if ($form->isSubmitted()) {

      //The form will only be valid if the checkbox was checked we therefore need to separate the submitted and valid logic
      //as the form may be submitted with an unchecked field (which makes it invalid) in which case the user must still
      //be able to progress to the next screen.
      if ($form->isValid()) {
        $entityManager = $this->get('doctrine.orm.entity_manager');
        $entityManager->persist($user);
        $entityManager->flush($user);
      }

      return $this->redirectToRoute('session_test');
    }

    $test = $this->getTest($testSession);
    $templateData['test'] = $test;
    $templateData['headerTitle'] = 'TUTORIAL';
    $templateData['form'] = $form->createView();

    return $this->render('@Website/Session/tutorial.html.twig', $templateData);
  }

  private function getTest($testSession)
  {
    //Get the test
    if (!empty($testSession)) {
      $testType = $testSession['type'];
    } else {
      $testType = Test::TYPE_PASAT;
    }

    $entityRepo = $this->get('doctrine.orm.entity_manager')->getRepository(Test::class);
    $results = $entityRepo->getTestByType($testType);
    return !empty($results) ? $results[0] : null;
  }

  public function testAction(Request $request)
  {
    try {
      $templateData = $this->getBasicData();
    } catch (AccessDeniedException $e) {
      return $this->handleAccessDeniedException($request, 'user_login');
    }

    $testSession = $templateData['testSession'];
    $experiment = $templateData['experiment'];
    $test = $this->getTest($testSession);

    if (empty($testSession)) {
      //If they're finished, send em back
      if ($this->getUser()->hasCompletedAllSessions()) {
        return $this->redirectToRoute('user_home');
      }

      $sessionDataService = $this->get('WebsiteBundle.Services.SessionDataService');
      $testSession = $sessionDataService->createNewFirstSession($this->getUser(), $experiment);
    }

    $templateData['test'] = $test;
    $templateData['testSession'] = $testSession;

    return $this->render('@Website/Session/test.html.twig', $templateData);
  }

  public function moodQuestionsAction(Request $request)
  {
    try {
      $experiment = $this->getExperiment();
    } catch (AccessDeniedException $e) {
      return $this->handleAccessDeniedException($request, 'user_login');
    }

    $moodQuestions = $experiment->getMoodQuestions();

    if (empty($moodQuestions)) {
      //If there are not mood questions just redirect straight to the reminder page
      return $this->redirectToRoute('session_reminder');
    }

    $templateData = [
      'questions' => $moodQuestions->toArray(),
      'headerTitle' => 'APASAT TRAINING'
    ];

    return $this->render('@Website/Session/moodQuestions.html.twig', $templateData);
  }

  public function reminderAction(Request $request)
  {
    try {
      $templateData = $this->getBasicData();
    } catch (AccessDeniedException $e) {
      return $this->handleAccessDeniedException($request, 'user_login');
    }

    $experiment = $templateData['experiment'];

    $participant = $this->getUser();
    $entityRepo = $this->get('doctrine.orm.entity_manager')->getRepository(TestSession::class);
    $lastSession = $entityRepo->getLastSession($participant, $experiment);

    //If they've reached this point and there is no last session then they have to start again
    if (empty($lastSession)) {
      $translator = $this->get('translator');
      $this->addFlash('error', $translator->trans('site.flash.session.noLastSession'));
      return $this->redirectToRoute('user_home');
    }

    //If they've not filled in the mood questions, send them back
    if ($lastSession->getNumber() != 1 && $lastSession->getMoodAnswers()->isEmpty()) {
      return $this->redirectToRoute('session_mood_questions');
    }

    $completed = $lastSession->getCompleted();

    $minDateTime = clone $completed;
    $minDateTime->setTimeZone(new \DateTimeZone('Europe/Brussels'));
    $minDateTime->modify('+' . $experiment->getMinTimeBetweenSession() . ' minutes');
    $maxDateTime = clone $completed;
    $maxDateTime->setTimeZone(new \DateTimeZone('Europe/Brussels'));
    $maxDateTime->modify('+' . $experiment->getMaxTimeBetweenSession() . ' minutes');

    $form = $this->createForm(ReminderType::class, null, [
      'minDateTime' => $minDateTime,
      'maxDateTime' => $maxDateTime
    ]);
    $form->handleRequest($request);

    //Ensure the $nextSession is between the min and max of the session range
    if ($form->isSubmitted() && $form->isValid()) {
      $nextSessionDateTime = $form->getData()['nextSession'];
      $sessionService = $this->get('WebsiteBundle.Services.SessionDataService');

      $nextSession = $sessionService->updateSession($participant, $experiment, $nextSessionDateTime);

      //Send the reminder email
      $emailService = $this->get('WebsiteBundle.services.EmailService');
      $emailService->sendReminderEmail($nextSession);

      return $this->redirectToRoute('user_statistics');
    } else if ($form->isSubmitted()) {
      $translator = $this->get('translator');

      $message = $translator->trans('site.reminder.errors.invalidDateTime', [
        '%minDateTime%' => $minDateTime->format('d/m H:i'),
        '%maxDateTime%' => $maxDateTime->format('d/m H:i')
      ]);

      $this->addFlash('error', $message);
    }

    $templateData = [
      'form' => $form->createView(),
      'experiment' => $experiment,
      'headerTitle' => 'PREVEND',
      'minDateTime' => $minDateTime,
      'maxDateTime' => $maxDateTime,
      'lastSession' => $lastSession
    ];

    return $this->render('@Website/Session/reminder.html.twig', $templateData);
  }

  public function redirectAction(Request $request)
  {

    $templateData = [
      'user' => $this->getUser()
    ];

    return $this->render('@Website/Session/tempSurveyRedirect.html.twig', $templateData);
  }
}
