<?php

namespace WebsiteBundle\Services\Dashboard;

use Bazookas\AdminBundle\AdminElements\Containers\AccordionContainer;
use Bazookas\AdminBundle\AdminElements\Containers\Container;
use Bazookas\AdminBundle\AdminElements\Containers\FlexContainer;
use Bazookas\AdminBundle\AdminElements\Containers\RowContainer;
use Bazookas\AdminBundle\AdminElements\Containers\TabbedContainer;
use Bazookas\AdminBundle\AdminElements\Elements\Actions\ListActions\EditRowActionElement;
use Bazookas\AdminBundle\AdminElements\Elements\Actions\ListActions\RemoveRowActionElement;
use Bazookas\AdminBundle\AdminElements\Elements\SmallStatisticElement;
use Bazookas\AdminBundle\AdminElements\Elements\TextElement;
use Bazookas\AdminBundle\AdminElements\Interfaces\AdminElementInterface;
use Bazookas\APIFrameworkBundle\Services\Data\Interfaces\DataServiceInterface;
use Doctrine\ORM\EntityManager;
use WebsiteBundle\AdminElements\Elements\ExperimentElement;
use WebsiteBundle\Entity\Experiment;
use WebsiteBundle\Entity\Researcher;
use WebsiteBundle\Repository\ExperimentRepository;

class HomeDashboardDataService implements DataServiceInterface
{

  /**
   * @var EntityManager
   */
  private $entityManager;

  /**
   * HomeDashboardDataService constructor.
   * @param EntityManager $entityManager
   */
  public function __construct(EntityManager $entityManager)
  {
    $this->entityManager = $entityManager;
  }

  /**
   * @param Researcher $user
   * @return AdminElementInterface[]
   */
  public function getDashboardElements(Researcher $user)
  {
    $entityRepo = $this->entityManager->getRepository(Experiment::class);

    $draftPanels = $this->getDraftExperimentsPanels($entityRepo, $user);
    $currentPanels = $this->getCurrentExperimentsPanels($entityRepo, $user);
    $inactivePanels = $this->getInactiveExperimentsPanels($entityRepo, $user);

    $allPanels = new AccordionContainer([
      new Container([$currentPanels], [
        'navName' => 'admin.dashboards.home.tabs.current.navName',
      ]),
      new Container([$draftPanels], [
        'navName' => 'admin.dashboards.home.tabs.draft.navName',
      ]),
      new Container([$inactivePanels], [
        'navName' => 'admin.dashboards.home.tabs.inactive.navName',
      ]),
    ], [
      'navName' => 'admin.dashboards.home.tabs.all.navName',
    ]);

    $panelContainer = new TabbedContainer([
      $allPanels,
      $draftPanels,
      $currentPanels,
      $inactivePanels,
    ], [
      'headerLabel' => 'admin.dashboards.home.panelHeader',
      'tabOrientation' => TabbedContainer::ORIENTATION_HORIZONTAL,
      'tabLocation' => TabbedContainer::LOCATION_LEFT,
    ]);

    return [$panelContainer];
  }

  /**
   * @param ExperimentRepository $repository
   * @param Researcher $user
   * @return Container|RowContainer
   */
  private function getDraftExperimentsPanels(ExperimentRepository $repository, Researcher $user)
  {
    $experiments = $repository->getDraftExperimentsForUser($user);
    return $this->createExperimentsPanels($experiments, 'admin.dashboards.home.tabs.draft.navName', [
      'actions' => [
        new EditRowActionElement([], [
          'route' => 'admin_experiments',
        ]),
        new RemoveRowActionElement([], [
          'route' => 'admin_experiments',
        ]),
      ],
    ]);
  }

  /**
   * @param Experiment[] $experiments
   * @param $navKey
   * @param array $options
   * @return Container
   */
  private function createExperimentsPanels(array $experiments, $navKey, array $options = [])
  {
    $panels = [];
    foreach ($experiments as $experiment) {
      $participantCount = $experiment->getParticipantCount();
      $activeCount = $experiment->getActiveParticipantCount();
      $inactiveCount = $participantCount - $activeCount;
      $stats = $this->createStatisticsWidgets($experiment->getSessionCount(), $participantCount, $activeCount, $inactiveCount);

      $options = array_merge($options, [
        'label' => $experiment->getName(),
        'entity' => $experiment,
        'researchers' => $experiment->getResearchers(),
      ]);

      $panels[] = new ExperimentElement([$stats], $options);
    }

    if (empty($panels)) {
      $panels[] = new TextElement([], [
        'label' => 'admin.dashboards.home.tabs.labels.noExperiments',
      ]);
    }

    $rows = [];
    for ($i = 0; $i < count($panels); $i++) {
      if ($i > 0 && ($i + 1) % 2 === 0) {
        $rows[] = new RowContainer([$panels[$i - 1], $panels[$i]], [
          'defaultColClass' => 'col-md-6 pad-top',
        ]);
      }
    }

    if (count($panels) % 2 > 0) {
      $rows[] = new RowContainer([$panels[count($panels) - 1]], [
        'defaultColClass' => 'col-md-6 pad-top',
      ]);
    }

    return new Container($rows, [
      'navName' => $navKey,
    ]);
  }

  /**
   * @param $sessionCount
   * @param $participantCount
   * @param $activeParticipants
   * @param $inactiveParticipants
   * @return FlexContainer
   */
  private function createStatisticsWidgets($sessionCount, $participantCount, $activeParticipants, $inactiveParticipants)
  {
    return new FlexContainer([
      new SmallStatisticElement([], [
        'value' => $sessionCount,
        'label' => 'admin.dashboards.home.tabs.labels.sessionCount',
        'icon' => ' fa-clock-o',
        'panelColor' => 'info',
      ]),
      new SmallStatisticElement([], [
        'value' => $participantCount,
        'label' => 'admin.dashboards.home.tabs.labels.participantCount',
        'icon' => 'fa-users',
        'panelColor' => 'primary',
      ]),
      new SmallStatisticElement([], [
        'value' => $activeParticipants,
        'label' => 'admin.dashboards.home.tabs.labels.activeParticipants',
        'icon' => 'fa-user',
        'panelColor' => 'success',
      ]),
      new SmallStatisticElement([], [
        'value' => $inactiveParticipants,
        'label' => 'admin.dashboards.home.tabs.labels.inactiveParticipants',
        'icon' => 'fa-user-o',
        'panelColor' => 'danger',
      ]),
    ]);
  }

  /**
   * @param ExperimentRepository $repository
   * @param Researcher $user
   * @return Container|RowContainer
   */
  private function getCurrentExperimentsPanels(ExperimentRepository $repository, Researcher $user)
  {
    $experiments = $repository->getCurrentExperimentsForUser($user);
    return $this->createExperimentsPanels($experiments, 'admin.dashboards.home.tabs.current.navName');
  }

  /**
   * @param ExperimentRepository $repository
   * @param Researcher $user
   * @return Container|RowContainer
   */
  private function getInactiveExperimentsPanels(ExperimentRepository $repository, Researcher $user)
  {
    $experiments = $repository->getInactiveExperimentsForUser($user);
    return $this->createExperimentsPanels($experiments, 'admin.dashboards.home.tabs.inactive.navName');
  }
}
