<?php

namespace Bazookas\AdminBundle\Controller\AdminController;

use Bazookas\AdminBundle\AdminElements\Elements\Actions\ListActions\NotificationGoToLinkRowActionElement;
use Bazookas\AdminBundle\AdminElements\Elements\Actions\ListActions\RemoveBulkActionElement;
use Bazookas\AdminBundle\AdminElements\Elements\Actions\ListActions\ViewedBulkActionElement;
use Bazookas\AdminBundle\AdminElements\Elements\Actions\ListActions\ViewRowActionElement;
use Bazookas\AdminBundle\Controller\Base\BaseAdminListController;
use Bazookas\AdminBundle\Entity\Interfaces\Notification\CMSNotificationInterface;
use Bazookas\AdminBundle\PageBuilder\Base\BaseFormPageBuilder;
use Bazookas\AdminBundle\PageBuilder\Interfaces\ListPageBuilderInterface;
use Bazookas\AdminBundle\PageBuilder\ListPageBuilder;
use Bazookas\AdminBundle\Repository\Interfaces\Notification\CMSNotificationRepositoryInterface;
use Bazookas\CommonBundle\Security\Roles;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;

class NotificationAdminController extends BaseAdminListController
{

  const ACTION_BULK_VIEWED = 'bulkViewed';
  const ACTION_REDIRECT = 'redirect';
  const ACTION_BULK_REMOVE = 'bulkRemove';

  /**
   * @inheritdoc
   */
  protected function modifyListBuilder(Request $request, ListPageBuilderInterface $builder)
  {
    /**
     * @var ListPageBuilder $builder
     */
    $builder = parent::modifyListBuilder($request, $builder);

    //Overwrite the entity name as we want the translations for this component to be standardised regardless of the projects notifications class
    $builder->setEntityShortName($this->getEntityShortClassName());

    $user = $this->getUser();
    $builder->setListQueryParams([
      'user' => $user,
    ]);

    $builder
      ->addBooleanField('new', [
        'label' => 'admin.entities.notification.fields.new',
      ])
      ->addField('message', [
        'template' => 'BazookasAdminBundle:List/Fields/Notification:_message.html.twig',
        'label' => 'admin.entities.notification.fields.message',
      ])
      ->addDateTimeField('created', [
        'label' => 'admin.entities.notification.fields.created',
      ]);

    $route = $request->get('_route');
    if ($this->hasAccess(self::ACTION_BULK_VIEWED)) {
      $builder->prependAction(new ViewedBulkActionElement([], [
        'route' => $route,
      ]));
    }

    $builder->addAction(new ViewRowActionElement([], [
      'action' => NotificationAdminController::ACTION_REDIRECT,
      'class' => 'btn-hover-info ti-direction',
      'label' => 'admin.actions.goToNotification',
      'route' => $route,
    ]));

    return $builder;
  }

  /**
   * @inheritdoc
   */
  public function getEntityShortClassName()
  {
    return 'notification';
  }

  /**
   * @inheritdoc
   */
  protected function hasAccess($action)
  {
    switch ($action) {
      case self::ACTION_BULK_VIEWED:
      case self::ACTION_BULK_REMOVE:
      case self::ACTION_LIST:
      case self::ACTION_REDIRECT:
        return $this->isGranted(Roles::ROLE_CMS_USER);
      default:
        return false;
    }
  }

  /**
   * @inheritdoc
   */
  protected function modifyAddBuilder(Request $request, BaseFormPageBuilder $builder)
  {
    $builder = parent::modifyAddBuilder($request, $builder);

    //Overwrite the entity name as we want the translations for this component to be standardised regardless of the projects notifications class
    $builder->setEntityShortName($this->getEntityShortClassName());

    return $builder;
  }

  protected function modifyEditBuilder(Request $request, BaseFormPageBuilder $builder)
  {
    $builder = parent::modifyEditBuilder($request, $builder);

    //Overwrite the entity name as we want the translations for this component to be standardised regardless of the projects notifications class
    $builder->setEntityShortName($this->getEntityShortClassName());

    return $builder;
  }

  /**
   * @inheritdoc
   */
  protected function getFormClass($action)
  {
    switch ($action) {
      default:
        return null;
    }
  }

  /**
   * @inheritdoc
   */
  protected function bulkViewedAction(Request $request, $action)
  {
    //Set the entities as viewed
    $entityIds = $request->get('ids');
    $entityRepo = $this->get('doctrine.orm.entity_manager')->getRepository($this->getEntityClass());
    if (!$entityRepo instanceof CMSNotificationRepositoryInterface) {
      throw new \Exception('Your entity class implements \'' . CMSNotificationInterface::class . '\' however its repository does not implement \'' . CMSNotificationRepositoryInterface::class . '\'');
    }

    $user = $this->getUser();
    $entityRepo->markNotificationsAsViewed($user, $entityIds);

    return $this->redirectToRoute($request->get('_route'));
  }

  /**
   * @inheritdoc
   */
  protected function getEntityClass()
  {
    $entities = $this->container->getParameter('bazookas_admin.notifications.entities');

    return $entities['CMSNotification'];
  }

  /**
   * Redirect the user to the link provided in the notification while also marking the notification as viewed
   * @param Request $request
   * @param $action
   * @return \Symfony\Component\HttpFoundation\RedirectResponse
   * @throws \Exception
   */
  protected function redirectAction(Request $request, $action)
  {
    $cmsNotificationRepository = $this->get('doctrine.orm.entity_manager')->getRepository($this->getEntityClass());

    if (!$cmsNotificationRepository instanceof CMSNotificationRepositoryInterface) {
      throw new \Exception('The CMSNotification repository must implement \'' . CMSNotificationRepositoryInterface::class . '\'');
    }

    $notificationId = $request->get('_id');
    $user = $this->getUser();

    $notification = $cmsNotificationRepository->findOneBy([
      'id' => $notificationId,
      'user' => $user,
    ]);

    if (!empty($notification) && $notification instanceof CMSNotificationInterface) {
      if ($notification->isNew()) {
        $cmsNotificationRepository->markNotificationsAsViewed($user, [$notificationId]);
      }

      return $this->redirect($notification->getLink());
    } else {
      throw new NotFoundHttpException();
    }
  }

  /**
   * @inheritdoc
   */
  protected function bulkRemoveAction(Request $request, $action)
  {
    $cmsNotificationRepository = $this->get('doctrine.orm.entity_manager')->getRepository($this->getEntityClass());

    if (!$cmsNotificationRepository instanceof CMSNotificationRepositoryInterface) {
      throw new \Exception('The CMSNotification repository must implement \'' . CMSNotificationRepositoryInterface::class . '\'');
    }

    $entityIds = $request->get('ids');

    if (!empty($entityIds)) {
      $user = $this->getUser();
      $cmsNotificationRepository->removeNotifications($user, $entityIds);
    }

    return $this->redirectToRoute('admin_notification');
  }
}
