<?php

namespace Bazookas\AdminBundle\EventListener\Entity;

use Bazookas\AdminBundle\Entity\Interfaces\AuditableInterface;
use Bazookas\AdminBundle\Entity\Interfaces\AuditLogInterface;
use Doctrine\ORM\Event\LifecycleEventArgs;
use Doctrine\ORM\Event\OnFlushEventArgs;
use Doctrine\ORM\UnitOfWork;
use Symfony\Component\Security\Core\Authentication\Token\Storage\TokenStorage;

class AuditLogListener {

  const ACTION_INSERT = 'insert';
  const ACTION_UPDATE = 'update';
  const ACTION_REMOVE = 'remove';

  /**
   * @var AuditLogInterface $logClass
   */
  private $logClass;
  private $tokenStorage;

  public function __construct($logClass, TokenStorage $tokenStorage) {
    if (!class_exists($logClass)) {
      throw new \InvalidArgumentException('The class \''.$logClass.'\' does not exist.');
    }

    $this->logClass = $logClass;
    $this->tokenStorage = $tokenStorage;
  }

  public function prePersist(LifecycleEventArgs $args) {
    $object = $args->getObject();
    if ($object instanceof AuditableInterface) {
      $token = $this->tokenStorage->getToken();
      $user = null;
      if (!empty($token)) {
        $user = $token->getUser();
      }

      $object->setCreator($user);
    }
  }

  public function onFlush(OnFlushEventArgs $args) {
    $em = $args->getEntityManager();
    $uow = $em->getUnitOfWork();

    //check updates
    foreach ($uow->getScheduledEntityUpdates() as $entity) {
      if ($entity instanceof AuditableInterface) {
        $logItem = $this->createActionLog(self::ACTION_UPDATE, $uow, $entity);
        $em->persist($logItem);
        $uow->computeChangeSet($em->getClassMetadata($this->logClass), $logItem);
      }
    }

    //check removals
    foreach ($uow->getScheduledEntityDeletions() as $entity) {
      if ($entity instanceof AuditableInterface) {
        $logItem = $this->createActionLog(self::ACTION_REMOVE, $uow, $entity);
        $em->persist($logItem);
        $uow->computeChangeSet($em->getClassMetadata($this->logClass), $logItem);
      }
    }
  }

  private function createActionLog($action, UnitOfWork $unitOfWork, AuditableInterface $entity) {
    $token = $this->tokenStorage->getToken();
    $user = null;
    if (!empty($token)) {
      $user = $token->getUser();
    }

    $logItem = new $this->logClass();
    $logItem
      ->setAction($action)
      ->setEntityName(get_class($entity))
      ->setDate(new \DateTime('now', new \DateTimeZone('UTC')))
      ->setUser($user)
    ;

    switch($action) {
      case self::ACTION_UPDATE:
        $logItem
          ->setEntityId($entity->getId())
          ->setData($unitOfWork->getEntityChangeSet($entity))
        ;
        break;
      case self::ACTION_REMOVE:
        $logItem
          ->setEntityId($entity->getId())
          ->setData($entity)
        ;
        break;
    }

    return $logItem;
  }
}
