<?php

namespace Bazookas\AdminBundle\Filter;

use Bazookas\AdminBundle\Filter\Filter;
use Bazookas\AdminBundle\Form\FilterForm;
use Bazookas\AdminBundle\Util\ListConfigurator;
use Doctrine\ORM\QueryBuilder;
use Symfony\Component\HttpFoundation\Request;

/**
 * @deprecated
 *
 * Class FilterService
 * @package Bazookas\AdminBundle\Filter
 */
class FilterService {

  protected $filters;
  private $config;

  public function __construct(ListConfigurator $config) {
    $this->filters = [];
    $this->config = $config;
  }

  public function getFilters() {
    return $this->filters;
  }

  public function addFilter($fieldName, Filter $filter) {
    $this->filters[$fieldName] = $filter;
    return $this;
  }

  public function hasFilter($fieldName) {
    return isset($this->filters[$fieldName]);
  }

  public function getFilter($fieldName) {
    if ($this->hasFilter($fieldName)) {
      return $this->filters[$fieldName];
    }
    return null;
  }

  /**
   * This function parses the filters from the request object
   * @param Request $request
   **/
  public function parseFilters(Request $request) {
    $route = $request->get('_route');
    $action = $request->get('_action');
    $session = $request->getSession();
    $filterHash = md5($route . $action);

    $clear = $request->query->get('clearFilters');

    if (!isset($clear)) {
      //handle any other filters passed in the url
      $filters = $request->query->get('filter', array());
      $filterForm = $request->query->get('filter_form', array());

      //clear any existing filters
      $this->filters = [];

      if (!empty($filterForm)) {
        $filters[] = $filterForm;
      }

      foreach ($filters as $filter) {
        $filterObject = new Filter();
        $filterObject->setField($filter['field']);
        $filterObject->setOperator($filter['operator']);
        if (isset($filter['value'])) {
          $filterObject->setValue($filter['value']);
        }
        else {
          $filterObject->setValue(0);
        }

        $this->filters[$filter['field']] = $filterObject;
      }

      if (empty($this->filters) && $this->config->getMaintainFilterState()) {
        $sessionFilters = $session->get('filters');

        if (isset($sessionFilters[$filterHash])) {
          $this->filters = $sessionFilters[$filterHash];
        } else if (!empty($sessionFilters)) {
          $session->remove('filters');
        }
      }

      //overwrite the session storage, this will remove any other filters still present
      $session->set('filters', [$filterHash => $this->filters]);
    } else {
      $this->filters = [];
      $session->set('filters', [$filterHash => $this->filters]);
    }
  }

  /**
   * This function parses the order given by the query parameters
   * @param Request $request
   **/
  public function parseOrder($request) {
    $order = $request->get('order', '');
    if (empty($order)) {
      return;
    }
    $order = explode(':', $order);
    $field = $this->config->getFieldByName($order[0]);
    if (empty($field) || !$field->sortable) {
      return;
    }
    $this->config->orderBy = $order[0];

    if (count($order) > 1) {
      $order = $order[1];
      // orderDirection is set to ASC by default
      // so no need to overwrite it if not desc
      if (strtoupper($order) === 'ASC') {
        $this->config->setOrderDirection('ASC');
      }
      else {
        $this->config->setOrderDirection('DESC');
      }
    }
  }
  /**
   * This function executes the filters
   * @param queryBuilder $queryBuilder the querybuilder instance to apply filters to
   **/
  public function executeFilters(QueryBuilder &$queryBuilder) {
    //Example of filter url
    //http://localhost:8001/en/admin/lessonplan/list?filter[0][field]=details.title&filter[0][operator]=like&filter[0][value]=test

    $filterParameter = 1;
    foreach ($this->filters as &$filter) {

      if (is_object($filter)) {
        $field = $this->joinAndGetFieldName($queryBuilder, $filter->getField());
        $operator = $filter->getOperator();
        $value = $filter->getValue();
      } else {
        $field = $this->joinAndGetFieldName($queryBuilder, $filter['field']);
        $operator = $filter['operator'];
        $value = $filter['value'];
      }

      if ($value === null ) {
        $operator = 'isNull';
      }

      if ($operator === 'like' || $operator === 'notLike') {
        $value = "%$value%";
      }

      $queryBuilder->andWhere(
        $queryBuilder->expr()->{$operator}($field, ":filter$filterParameter")
      );
      if ($operator != 'isNull') {
        $queryBuilder->setParameter(":filter$filterParameter", $value);
      }

      $filterParameter++;
    }
  }

  public function executeOrder(QueryBuilder $queryBuilder) {
    $orderField = $this->config->getFieldByName($this->config->orderBy);
    $orderDirection = $this->config->orderDirection;
    if (empty($orderField)) {
      $queryBuilder->orderBy('e.' . $this->config->orderBy, $orderDirection);
      return;
    }

    $orderBy = ($orderField->sortable === true ? $orderField->sortable : $this->config->orderBy);
    $orderBy = $this->joinAndGetFieldName($queryBuilder, $orderBy);
    $queryBuilder->orderBy($orderBy, $orderDirection);
  }

  private function joinAndGetFieldName(QueryBuilder $queryBuilder, $fieldPath) {
    $exploded = explode('.', $fieldPath);
    if (count($exploded) === 1) {
      return 'e.' . $fieldPath;
    }
    else {
      $lastField = array_pop($exploded);
      $previousField = 'e';
      foreach ($exploded as $field) {
        if (!$this->joinExsists($queryBuilder, $previousField, $field)) {
          $queryBuilder->leftJoin("$previousField.$field", $field);
        }
        $previousField = $field;
      }
      return "$field.$lastField";
    }
  }

  private function joinExsists(QueryBuilder $queryBuilder, $prefix, $alias) {
    $parts = $queryBuilder->getDQLPart('join');
    if (isset($parts[$prefix])) {
      foreach ($parts[$prefix] as $existingJoin) {
        if ($existingJoin->getAlias() === $alias) {
          return true;
        }
      }
    }
    return false;
  }

  public function createFilterForm($formFactory) {
    if (!empty($this->config->listFilters)) {
      if (!empty($this->filters)) {
        $filter = $this->filters[array_keys($this->filters)[0]];
      } else {
        $filter = new Filter();
      }

      $filter->setFilterFields($this->config->listFilters);

      $form = $formFactory
        ->createNamedBuilder('filter_form', FilterForm::class, $filter, array(
          'method' => 'GET'
        ))
        ->getForm()
      ;
      return $form->createView();
    } else {
      return null;
    }
  }
}
