### Adding notifications to the cms

In order to make use of the cms's built in notification system you must first define some entities for  the system to use.

#### Creating entities

Start by defining a `BaseNotification` class. The base notification class should make use of doctrine's single table inheritance to reduce the number of tables in the database. Do note that this **is** required--your entity **must** use doctrine inheritance. Also be aware that the notifications system currently only supports one level of inheritance. Meaning: you may only extend a maximum of 1 mapped entities (You must always directly extend your `BaseNotification` class).

Your base notification class could as follows. Note that you will need to set the relation with the CMSUser yourself.

```php
<?php

namespace WebsiteBundle\Entity\Base;

use Bazookas\CommonBundle\Entity\Base\BaseCMSUser;
use Doctrine\ORM\Mapping as ORM;
use \Bazookas\AdminBundle\Entity\Base\BaseNotification as AdminBaseNotification;

/**
 * @ORM\Entity(repositoryClass="WebsiteBundle\Repository\Base\BaseNotificationRepository")
 * @ORM\Table(name="notifications")
 * @ORM\InheritanceType("SINGLE_TABLE")
 * @ORM\DiscriminatorColumn(name="type", type="string")
 * @ORM\HasLifecycleCallbacks
 */
class BaseNotification extends AdminBaseNotification {

  /**
   * @var BaseCMSUser
   * @ORM\ManyToOne(targetEntity="Bazookas\AdminBundle\Entity\CMSUser")
   */
  protected $user;

}
```

And might make use of a repository which could be defined as follows:

```php
<?php

namespace WebsiteBundle\Repository\Base;

use Bazookas\AdminBundle\Repository\Base\BaseNotificationRepository as AdminBaseNotificationRepository;

class BaseNotificationRepository extends AdminBaseNotificationRepository
{
 
}
```

Once you've done this basic setup you can now add your own notification classes. These notifications classes will define the **type** of notification that will be sent. The AdminBundle currently supports these notifications out-of-the-box:

- `CMSNotifications` notifications which are only shown in the cms interface.
- `EmailNotifications` notifications which are sent by email.

##### Adding CMSNotifications

Let's continue by implementing basic `CMSNotifications`. First create a `CMSNotification` entity in your project, it's code might look as follows:

```php
<?php

namespace WebsiteBundle\Entity;

use Bazookas\AdminBundle\Entity\Interfaces\Notification\CMSNotificationInterface;
use Bazookas\AdminBundle\Entity\Traits\Notification\CMSNotificationTrait;
use Doctrine\ORM\Mapping as ORM;
use WebsiteBundle\Entity\Base\BaseNotification;

/**
 * @ORM\Entity(repositoryClass="Bazookas\AdminBundle\Repository\Base\BaseCMSNotificationRepository")
 * @ORM\HasLifecycleCallbacks
 */
class CMSNotification extends BaseNotification implements CMSNotificationInterface {

  use CMSNotificationTrait;

}
```

The admin bundle provides a basic implementation of a CMSNotificationRepository. You can use this class directly or extend it to add extra functionality.

##### Adding EmailNotifications

Create your `EmailNotification` class:

```php
<?php

namespace WebsiteBundle\Entity;

use Bazookas\AdminBundle\Entity\Interfaces\Notification\EmailNotificationInterface;
use Bazookas\AdminBundle\Entity\Traits\Notification\EmailNotificationTrait;
use Doctrine\ORM\Mapping as ORM;
use Symfony\Component\Validator\Constraints as Assert;
use WebsiteBundle\Entity\Base\BaseNotification;

/**
 * @ORM\Entity(repositoryClass="Bazookas\AdminBundle\Repository\Base\BaseEmailNotificationRepository")
 * @ORM\HasLifecycleCallbacks
 */
class EmailNotification extends BaseNotification implements EmailNotificationInterface {

  use EmailNotificationTrait;

}
```

The admin bundle provides a basic implementation of an EmailNotificationRepository. You can use this class directly or extend it to add extra functionality.

**NOTE**: When using email notifications make sure to ALWAYS use absolute urls.

##### Final entity config

Once you've created your entities you will need to perform a last bit of config to register them with the notification system. Add the following to `config.yml`:

```yaml
bazookas_admin:
  notifications:
    notificationEntities:
      CMSNotification: WebsiteBundle\Entity\CMSNotification
      EmailNotification: WebsiteBundle\Entity\EmailNotification
```

And enable the cms notification in the admin bundle layout by setting the twig global:
```yaml
twig:
    globals:
      enableCMSNotifications: true
```

Finally, if you are using the `EmailNotification` functionality, don't forget to add a cron job for the actual sending of the emails! The cron job will bundle all email notifications into a single mail.

```
bin/console notifications:email:send --env=prod
```

#### Start generating notifications

Now that we've setup the notification entities and done the last bit of configuration we need to program where to add the notifications. The circumstances which cause a notification to be generated are project specific and thus you will need to implement this aspect of the notification system. We've created the `NotificationService` to help you with this.

In essence what it comes down to is that **any time** you wish to add a notification you should make use of the `NotificationService::addNotification()` function. You may use this service anywhere to add new notifications. For example you might want to generate notification from one (or all) of these locations:

1. Using a doctrine event listener to listen to Doctrine lifecycle events
2. Using a form event in a Symfony form
3. In the controller action of your page.

This next section will show you how to create a doctrine event listener.

##### Notification event listener sample implementation

Start by creating a `NotificationListener`:

```php
<?php

namespace WebsiteBundle\EventListener;

use Bazookas\AdminBundle\Services\Notification\NotificationService;
use Doctrine\ORM\Event\LifecycleEventArgs;
use Symfony\Component\Routing\Router;
use WebsiteBundle\Controller\Admin\ClaimableIncentiveAdminController;
use WebsiteBundle\Controller\Admin\VehicleAdminController;
use WebsiteBundle\Entity\ClaimableIncentive;
use WebsiteBundle\Entity\CMSNotification;
use WebsiteBundle\Entity\Vehicle;
use WebsiteBundle\Security\Roles;

class NotificationListener
{

  /**
   * @var NotificationService
   */
  private $notificationService;

  /**
   * @var Router
   */
  private $router;

  public function __construct(NotificationService $notificationService, Router $router) {
    $this->notificationService = $notificationService;
    $this->router = $router;
  }

  /**
   * @param LifecycleEventArgs $args
   */
  public function postPersist(LifecycleEventArgs $args) {
    $object = $args->getObject();

    //We should add a notification when a vehicle is first inserted
    if ($object instanceof Vehicle) {
      $this->addVehicleNotification($args);
    }
    
    if ($object instanceof ClaimableIncentive) {
      $this->addVendorIncentiveNotification($args);
    }
  }

  /**
   * @param LifecycleEventArgs $args
   */
  private function addVehicleNotification(LifecycleEventArgs $args) {
    $object = $args->getObject();
    $link = $this->router->generate('admin_vehicle', [
      '_action' => VehicleAdminController::ACTION_VIEW,
      '_id' => $object->getId()
    ]);

    $notification = new CMSNotification();
    $notification
      ->setMessage('admin.notifications.newVehicle')
      ->setIcon('fa fa-2x fa-car')
      ->setLink($link)
    ;

    $this->notificationService->addNotification($notification, [Roles::ROLE_ACCOUNT_MANAGER]);
  }

  /**
   * @param LifecycleEventArgs $args
   */
  private function addVendorIncentiveNotification(LifecycleEventArgs $args) {
    $link = $this->router->generate('admin_claimable_incentive', [
      '_action' => ClaimableIncentiveAdminController::ACTION_LIST
    ]);

    $notification = new CMSNotification();
    $notification
      ->setMessage('admin.notifications.newIncentive')
      ->setIcon('fa fa-2x fa-money')
      ->setLink($link)
    ;

    $this->notificationService->addNotification($notification, [Roles::ROLE_ACCOUNTING]);
  }

}
```

Don't forget to define your listener in `services.yml`:

```yaml
  admin.notifications.notificationListener:
    class: WebsiteBundle\EventListener\NotificationListener
    arguments:
      - "@bazookas.admin.service.NotificationService"
      - "@router"
    tags:
      - { name: doctrine.event_listener, event: postPersist }
```

Using the event listener you can catch any post persist events for any entity. The notification service will take care of doing any special actions with the notification. In the case of a CMSNotification this is simply persisting the entity. In the case of the EmailNotification this also takes care of sending the actual email.
