<?php

namespace Bazookas\AdminBundle\Services\Notification;

use Bazookas\AdminBundle\Entity\CMSUser;
use Bazookas\AdminBundle\Entity\Interfaces\Notification\CMSNotificationInterface;
use Bazookas\AdminBundle\Entity\Interfaces\Notification\EmailNotificationInterface;
use Bazookas\AdminBundle\Entity\Interfaces\Notification\NotificationInterface;
use Bazookas\AdminBundle\Repository\Interfaces\CMSUserRepositoryInterface;
use Bazookas\AdminBundle\Repository\Interfaces\Notification\CMSNotificationRepositoryInterface;
use Bazookas\CommonBundle\Entity\Base\BaseCMSUser;
use Bazookas\CommonBundle\Exception\NotImplementedException;
use Doctrine\ORM\EntityManagerInterface;
use Doctrine\ORM\Mapping\MappingException;
use Doctrine\ORM\Query;

class NotificationService
{
  /**
   * @var EntityManagerInterface
   */
  private $entityManager;

  /**
   * @var array
   */
  private $notificationEntities;

  /**
   * @var string
   */
  private $cmsNotificationEntityClass;

  /**
   * @var array
   */
  private $roleHierarchy;

  public function __construct(EntityManagerInterface $entityManager, array $notificationEntities, array $roleHierarchy) {
    $this->entityManager = $entityManager;
    $this->notificationEntities = $notificationEntities;
    $this->roleHierarchy = $roleHierarchy;
  }

  /**
   * @param BaseCMSUser $user
   * @return NotificationInterface[]
   * @throws \Exception
   */
  public function getCMSNotificationsForUser(BaseCMSUser $user) {
    $entityClass = $this->getCMSNotificationEntityClass();

    $notificationRepo = $this->entityManager->getRepository($entityClass);

    if (!$notificationRepo instanceof CMSNotificationRepositoryInterface) {
      throw new \Exception('Your entity class implements \''.CMSNotificationInterface::class.'\' however its repository does not implement \''.CMSNotificationRepositoryInterface::class.'\'');
    }

    return $notificationRepo->getCMSNotifications($user);
  }

  /**
   * Get all roles and all inherited roles for a given cms user
   * @param BaseCMSUser $user
   * @return array
   */
  public function getRolesForUser(BaseCMSUser $user)
  {
    $userRoles = $user->getRoles();

    for ($i = 0; $i < count($userRoles); $i++) {
      $userRole = $userRoles[$i];
      if (isset($this->roleHierarchy[$userRole])) {
        $userRoles = array_merge($userRoles, $this->roleHierarchy[$userRole]);
      }
    }

    return array_unique($userRoles);
  }

  /**
   * @return string
   * @throws \Exception
   */
  private function getCMSNotificationEntityClass() {
    if (empty($this->cmsNotificationEntityClass)) {
      $entityClass = $this->notificationEntities['CMSNotification'];
      if (!isset($entityClass)) {
        throw new \Exception('No CMSNotification entity has been defined.');
      }

      try {
        $this->entityManager->getClassMetadata($entityClass);
      } catch (MappingException $e) {
        throw new \Exception('No CMSNotification entity has been defined.');
      }

      $reflectionClass = new \ReflectionClass($entityClass);
      if (!$reflectionClass->implementsInterface(CMSNotificationInterface::class)) {
        throw new \Exception('Your CMSNotification entity must implement \''.CMSNotificationInterface::class.'\'');
      }

      $this->cmsNotificationEntityClass = $entityClass;
    }

    return $this->cmsNotificationEntityClass;
  }

  /**
   * @param $roles
   * @return BaseCMSUser[]
   * @throws \Exception
   */
  private function getUsersForRoles($roles) {
    foreach($this->roleHierarchy as $parentRole => $inheritedRoles) {
      foreach($roles as $userRole) {
        if (in_array($userRole, $inheritedRoles)) {
          $roles[] = $parentRole;
        }
      }
    }

    $cmsUserRepository = $this->entityManager->getRepository(CMSUser::class);
    if (!$cmsUserRepository instanceof CMSUserRepositoryInterface) {
      throw new \Exception('Your Repository class must implement \''.CMSUserRepositoryInterface::class.'\'');
    }

    return $cmsUserRepository->getUsersByRoles($roles, Query::HYDRATE_OBJECT);
  }

  /**
   * Add a new notification.
   *
   * Note: when passing an array of roles in $applyToRoles the user property set on the original $notification will be
   * ignored as the notification will be added to all users with those roles
   * @param NotificationInterface $notification
   * @param array $applyToRoles
   * @throws \Exception
   */
  public function addNotification(NotificationInterface $notification, $applyToRoles = []) {
    if (!empty($applyToRoles)) {
      $users = $this->getUsersForRoles($applyToRoles);

      foreach($users as $user) {
        $temp = clone $notification;
        $temp->setUser($user);
        $this->entityManager->persist($temp);
      }
    } else {
      $this->entityManager->persist($notification);
    }

    $this->entityManager->flush();
  }

}
