<?php

namespace Bazookas\AdminBundle\Twig\Base;

use Bazookas\AdminBundle\AdminElements\Containers\MenuContainer;
use Bazookas\AdminBundle\AdminElements\Elements\Actions\MenuActions\MenuActionElement;
use Bazookas\AdminBundle\AdminElements\Elements\Actions\MenuActions\MenuHeaderElement;
use Bazookas\AdminBundle\AdminElements\Interfaces\AdminElementInterface;
use Bazookas\CommonBundle\Security\Roles;
use Symfony\Component\HttpFoundation\Request;

class BaseAdminMenuExtension extends \Twig_Extension
{

  /**
   * @var AdminElementInterface[]
   */
  protected $elements = [];

  /**
   * @var string[]
   */
  protected $bundles;

  /**
   * @deprecated to be removed in v2
   * @var array
   */
  protected $v1MenuConfig;

  public function __construct($bundles, $v1MenuConfig) {
    $this->bundles = $bundles;
    $this->v1MenuConfig = $v1MenuConfig;
  }

  /**
   * Get Twig functions defined in this extension.
   *
   * @return array
   */
  public function getFunctions()
  {
    return array(
      new \Twig_SimpleFunction('buildAdminMenu', array($this, 'buildAdminMenu'), [
        'needs_environment' => true
      ]),
    );
  }

  /**
   * @param \Twig_Environment $env
   * @return AdminElementInterface[]
   */
  public function buildAdminMenu(\Twig_Environment $env) {
    if (empty($this->elements)) {
      $app = $env->getGlobals()['app'];

      /** @var Request $request */
      $request = $app->getRequest();
      $this->createMenu($request);
    }

    return $this->elements;
  }

  /**
   * @param Request $request
   */
  protected function createMenu(Request $request) {
    $this->addMenuHeader();
    $this->addMediaBundleMenuItems();
    $this->addAdminSettingsMenu();
  }

  protected function addMenuHeader() {
    $this->elements[] = new MenuHeaderElement([], [
      'label' => 'admin.menu.admin.header'
    ]);
  }

  protected function addAdminSettingsMenu() {
    $adminMenuItems = $this->createAdminBundleMenuItems();
    $cronMenuItems = $this->createCronBundleMenuItems();

    /** @var AdminElementInterface[] $menuItems */
    $menuItems = array_merge($adminMenuItems, $cronMenuItems);

    //Create the roles array
    $roles = [];
    foreach ($menuItems as $menuItem) {
      $roles = $roles + $menuItem->getRoles();
    }

    $roles = array_unique($roles);

    //Add the menu container
    $settingsContainer = new MenuContainer($menuItems, [
      'label' => 'admin.menu.settings.label',
      'iconClass' => 'fa fa-gears',
      'roles' => $roles,
    ]);

    $this->elements[] = $settingsContainer;
  }

  /**
   * Convenience function to add just the admin bundle menu items
   */
  protected function addAdminBundleMenuItems() {
    $menuItems = $this->createAdminBundleMenuItems();

    foreach($menuItems as $item) {
      $this->elements[] = $item;
    }
  }

  /**
   * Create the admin bundle menu items
   * @return AdminElementInterface[]
   */
  protected function createAdminBundleMenuItems() {
    return [
      //Add the user management menu items
      new MenuContainer([
        new MenuActionElement([], [
          'label' => 'admin.entities.cMSUser.namePlural',
          'route' => 'admin_cms_user',
          'iconClass' => 'ti ti-user',
          'roles' => [Roles::ROLE_ADD_USER],
        ]),
        new MenuActionElement([], [
          'label' => 'admin.entities.cMSUserGroup.namePlural',
          'route' => 'admin_cms_user_group',
          'iconClass' => 'ti-id-badge',
          'roles' => [Roles::ROLE_ADD_GROUP],
        ])
      ],
      [
        'label' => 'admin.menu.users.label',
        'roles' => [Roles::ROLE_ADD_USER, Roles::ROLE_ADD_GROUP],
        'iconClass' => 'ti ti-user'
      ]),
      //Add the copy module menu items
      new MenuActionElement([], [
        'label' => 'admin.menu.copy.label',
        'route' => 'admin_copy',
        'iconClass' => 'fa fa-language',
        'roles' => [ Roles::ROLE_COPY_MANAGER ],
      ])
    ];
  }

  /**
   * Add and create the media bundle menu items
   * @param bool $includeVideoItems
   */
  protected function addMediaBundleMenuItems($includeVideoItems = false) {
    if (isset($this->bundles['BazookasMediaBundle'])) {
      $this->elements[] = $this->createMediaBundleMenuItems($includeVideoItems);
    }
  }

  /**
   * Convenience function
   * @param bool $includeVideoItems
   * @return MenuContainer
   */
  protected function createMediaBundleMenuItems($includeVideoItems = false) {
    $container = new MenuContainer([],
      [
        'label' => 'admin.menu.media',
        'roles' => [Roles::ROLE_MEDIA_ADMIN],
        'icon' => 'ti-image'
      ]
    );

    $container
      ->addChild(new MenuActionElement([], [
        'label' => 'admin.entities.image.namePlural',
        'route' => 'bazookas_media_image_admin',
        'icon' => 'ti-image',
        'roles' => [Roles::ROLE_MEDIA_ADMIN],
      ]))
    ;

    if ($includeVideoItems) {
      $container
        ->addChild(new MenuActionElement([], [
          'label' => 'admin.entities.video.namePlural',
          'route' => 'bazookas_media_video_admin',
          'iconClass' => 'ti-video-clapper',
          'roles' => [Roles::ROLE_MEDIA_ADMIN],
        ]))
        ->addChild(new MenuActionElement([], [
          'label' => 'admin.entities.videoProvider.namePlural',
          'route' => 'bazookas_media_video_provider_admin',
          'icon' => 'ti-settings',
          'roles' => [Roles::ROLE_MEDIA_ADMIN],
        ]))
      ;
    }

    $container
      ->addChild(new MenuActionElement([], [
        'label' => 'admin.entities.mediaFolder.namePlural',
        'route' => 'bazookas_media_media_folder_admin',
        'iconClass' => 'ti-folder',
        'roles' => [Roles::ROLE_MEDIA_ADMIN],
      ]))
    ;

    return $container;
  }

  /**
   * Add the cron bundle menu items
   */
  protected function addCronBundleMenuItems() {
    $menuItems = $this->createCronBundleMenuItems();

    foreach($menuItems as $item) {
      $this->elements[] = $item;
    }
  }

  /**
   * Create the cron bundle menu items
   * @return AdminElementInterface[]
   */
  protected function createCronBundleMenuItems() {
    if (isset($this->bundles['BazookasCronBundle'])) {
      return [
        new MenuActionElement([], [
          'label' => 'admin.entities.cron.namePlural',
          'route' => 'bazookas_cron_cron_admin',
          'iconClass' => 'ti-list',
          'roles' => [Roles::ROLE_CRON_ADMIN],
        ])
      ];
    } else {
      return [];
    }
  }

  /**
   * @deprecated to be removed in v2
   * This function adds backwards compatibility between the v1 and v2 menu structures.
   */
  protected function addV1MenuItems() {
    foreach($this->v1MenuConfig['main'] as $key => $config) {
      if ($config['type'] == 'submenu') {
        $element = new MenuContainer();

        foreach($config['children'] as $childConfig) {
          $childElement = new MenuActionElement();

          $childElement->setOptions([
            'label' => $childConfig['label'],
            'route' => $childConfig['route'],
            'iconClass' => $childConfig['icon'],
            'roles' => $childConfig['roles']
          ]);

          $element->addChild($childElement);
        }
      } else {
        $element = new MenuActionElement();
      }

      $options = [
        'label' => $config['label'],
        'iconClass' => $config['icon'],
        'roles' => $config['roles']
      ];

      if ($element instanceof MenuActionElement) {
        $options['route'] = $config['route'];
      }

      $element->setOptions($options);

      $this->addV1Element($element);
    }
  }

  /**
   * @deprecated to be removed in v2
   * @param AdminElementInterface $v1Element
   */
  private function addV1Element(AdminElementInterface $v1Element) {
    $addElement = true;
    foreach($this->elements as $element) {
      if ($v1Element instanceof MenuContainer) {
        if ($element instanceof MenuContainer && $v1Element->getLabel() == $element->getLabel()) {
          $addElement = false;
          break;
        }
      } else if ($element instanceof MenuActionElement && $element->getRoute() == $v1Element->getRoute()) {
        $addElement = false;
        break;
      }
    }

    if ($addElement) {
      $this->elements[] = $v1Element;
    }

  }
}
