<?php

namespace Bazookas\APIFrameworkBundle\DependencyInjection;

use Bazookas\APIFrameworkBundle\Form\AppCopyDetailAdminType;
use Bazookas\APIFrameworkBundle\Services\Data\AppCopyDataService;
use Bazookas\APIFrameworkBundle\Services\Data\AppSettingsDataService;
use Symfony\Component\Config\Definition\Exception\Exception;
use Symfony\Component\DependencyInjection\ContainerBuilder;
use Symfony\Component\Config\FileLocator;
use Symfony\Component\DependencyInjection\Extension\PrependExtensionInterface;
use Symfony\Component\HttpKernel\DependencyInjection\Extension;
use Symfony\Component\DependencyInjection\Loader;
use Symfony\Component\DependencyInjection\Reference;

/**
 * This is the class that loads and manages your bundle configuration.
 *
 * @link http://symfony.com/doc/current/cookbook/bundles/extension.html
 */
class BazookasAPIFrameworkExtension extends Extension implements PrependExtensionInterface
{
  /**
   * {@inheritdoc}
   */
  public function load(array $configs, ContainerBuilder $container)
  {
    $configuration = new Configuration();
    $config = $this->processConfiguration($configuration, $configs);
    $bundles = $container->getParameter('kernel.bundles');

    $loader = new Loader\YamlFileLoader($container, new FileLocator(__DIR__.'/../Resources/config'));
    $loader->load('services.yml');

    if (isset($config['file_upload'])) {
      $container->setParameter('bazookas_api.file_upload.settings', $config['file_upload']['settings']);
      $container
        ->register('BazookasAPIFrameworkBundle.upload.fileUploadService', 'Bazookas\APIFrameworkBundle\Services\Upload\APIFileUploadService')
        ->addArgument(new Reference('doctrine.orm.entity_manager'))
        ->addArgument(new Reference('router'))
        ->addArgument('%bazookas_api.file_upload.settings%')
        ->addMethodCall('setContainer', array(new Reference('service_container')))
        ->addTag('APIFrameworkBundle.Data.DataService')
      ;
    }

    if (isset($config['oAuth']) && $config['oAuth']['facebookGrantTypeEnabled']) {
      if (empty($config['oAuth']['facebookGrantTypeUserProvider'])) {
        throw new Exception('Missing required "bazookas_api_framework.oAuth.facebookGrantTypeUserProvider" configuration');
      }
      $container
        ->register('APIFrameworkBundle.oauth.facebook_grant_extension', 'Bazookas\APIFrameworkBundle\Security\OAuth\GrantTypeExtensions\FacebookOAuthGrantExtension')
        ->addArgument(new Reference($config['oAuth']['facebookGrantTypeUserProvider']))
        ->addArgument(new Reference('BazookasIntegrationsBundle.services.FacebookService'))
        ->addArgument($config['oAuth']['allowRegistration'])
        ->addTag('fos_oauth_server.grant_extension', array('uri' => 'http://facebook.com' ))
      ;
    }

    if (isset($config['app_settings'])) {
      $container
        ->register('APIFrameworkBundle.Data.AppSettingsDataService', AppSettingsDataService::class)
        ->addArgument(new Reference('doctrine.orm.entity_manager'))
        ->addArgument($config['app_settings']['entity'])
      ;

    }

    if (isset($config['app_copy'])) {
      // check if specific locales are set, otherwise get the default symfony locales
      if (isset($config['app_copy']['allowed_locales'])) {
        $locales = $config['app_copy']['allowed_locales'];
      }
      else {
        $locales = $container->getParameter('requiredlocales');
      }

      // set it as a parameter to allow use in routing
      $container->setParameter('bazookas_api_framework.app_copy.allowed_locales', $locales);
      $container->setParameter('bazookas_api_framework.app_copy.entity', $config['app_copy']['entity']);

      // and finally: register the dataservice
      $container
        ->register('APIFrameworkBundle.Data.AppCopyDataService', AppCopyDataService::class)
        ->addArgument(new Reference('doctrine.orm.entity_manager'))
        ->addArgument($config['app_copy']['entity'])
      ;

      if (isset($bundles['BazookasAdminBundle'])) {
        // and when the admin bundle is active register the details form and give it the correct locales and entity
        if (empty($config['app_copy']['details_entity'])) {
          throw new Exception('"bazookas_api_framework.app_copy.details_entity" configuration is required when BazookasAdminBundle is enabled.');
        }
        $container
          ->register('APIFrameworkBundle.Data.AppCopyDetailAdminType', AppCopyDetailAdminType::class)
          ->addArgument($locales)
          ->addArgument($config['app_copy']['details_entity'])
          ->addTag('form.type')
        ;
      }
    }
  }

  public function prepend(ContainerBuilder $container)
  {
    $bundles = $container->getParameter('kernel.bundles');

    if (isset($bundles['BazookasAdminBundle'])) {
      // load menu configurations for settings and copy
      $configs = $container->getExtensionConfig($this->getAlias());
      $config = $this->processConfiguration(new Configuration(), $configs);

      $menuConfig = [];
      if (isset($config['app_copy'])) {
        $this->addAppCopyMenu($container, $menuConfig);
      }

      if (isset($config['app_settings'])) {
        $this->addAppSettingsMenu($container, $menuConfig);
      }

      $container->prependExtensionConfig('bazookas_admin', $menuConfig);
    }
  }

  private function addAppCopyMenu(ContainerBuilder $container, &$menuConfig) {
    $menuConfig['menu']['main']['app_copy'] = [
      'type' => 'link',
      'icon' => 'ti-world',
      'label' => 'admin.entities.appCopy.namePlural',
      'route' => 'bazookas_api_framework_app_copy_admin',
      'roles' => ['ROLE_APP_COPY'],
    ];
  }

  private function addAppSettingsMenu(ContainerBuilder $container, &$menuConfig) {
    $menuConfig['menu']['main']['app_settings'] = [
      'type' => 'link',
      'icon' => 'ti-settings',
      'label' => 'admin.entities.appSettings.namePlural',
      'route' => 'bazookas_api_framework_app_settings_admin',
      'roles' => ['ROLE_APP_SETTINGS'],
    ];
  }

}
