<?php

namespace Bazookas\APIFrameworkBundle\Services\Validation;

use Bazookas\APIFrameworkBundle\Exception\Base\ApiExceptionMap;
use Bazookas\APIFrameworkBundle\Exception\InvalidArgumentException;
use Bazookas\APIFrameworkBundle\Exception\ServiceException;
use Symfony\Component\HttpFoundation\ParameterBag;
use Symfony\Component\OptionsResolver\OptionsResolver;
use Symfony\Component\Validator\Constraint;
use Symfony\Component\Validator\Validator\ValidatorInterface;

class ValidationService {

  //conversion types
  const TYPE_DATETIME = 'datetime';
  const TYPE_GENERIC = 'generic';
  /** @deprecated */
  const TYPE_STRING = self::TYPE_GENERIC;

  private $validator;
  private $resolver;

  public function __construct(ValidatorInterface $validator) {
    $this->validator = $validator;

    $this->resolver = new OptionsResolver();
    $this->resolver
      ->setDefaults([
        'constraint' => '',
        'required' => false,
        'convertToType' => self::TYPE_GENERIC,
        'conversionFormat' => '',
        'dependencies' => [],
        'excludes' => []
      ])
      ->setAllowedTypes('constraint', ['array', 'string', 'null', Constraint::class])
      ->setAllowedTypes('required', ['null', 'boolean'])
      ->setAllowedValues('convertToType', [
        self::TYPE_GENERIC,
        self::TYPE_DATETIME
      ])
      ->setAllowedTypes('conversionFormat', [
        'string',
        'callable'
      ])
      ->setAllowedTypes('dependencies', ['null', 'array', 'string'])
      ->setAllowedTypes('excludes', ['null', 'array', 'string'])
    ;
  }

  public function validate($fieldName, $value, $requirements) {
    $required = isset($requirements['required']) ? $requirements['required'] : false;

    if ($required && !isset($value)) {
      throw new InvalidArgumentException("'$fieldName': This field is required.", ApiExceptionMap::MISSING_FIELD);
    }

    if ( !isset($requirements['constraint']) || (!isset($value) && !$required)) {
      return $value;
    }

    //If an array of constraints is given, recursively call this function to
    //validate this value against each constraint.
    if (is_array($requirements['constraint'])) {
      foreach($requirements['constraint'] as $constraint) {
        $requirements['constraint'] = $constraint;
        $value = $this->validate($fieldName, $value, $requirements);
      }
    } else if ($requirements['constraint'] instanceof Constraint) {
      $value = $this->formatData($value, $requirements);

      $errorList = $this->validator->validate($value, $requirements['constraint']);
      if (count($errorList) > 0) {
        throw new InvalidArgumentException("'$fieldName{$errorList[0]->getPropertyPath()}': ".$errorList[0]->getMessage());
      }
    } else if (!empty($requirements['constraint'])) {
      throw new ServiceException("Field '".$fieldName. "'is bound by an unknown constraint: '".print_r($requirements['constraint'], true)."'");
    }

    return $value;
  }

  private function validateFields(ParameterBag &$parameterBag, array $requirements, array $data) {
    foreach($requirements as $key => $requirement) {
      $value = isset($data[$key]) ? $data[$key] : null;
      $required = isset($requirement['required']) ? $requirement['required'] : false;

      $value = $this->validate($key, $value, $requirement);

      if ($required || isset($value)) {
        $parameterBag->add(array($key => $value));
      }
    }
  }

  private function validateDependencies(array &$requirements, array $data) {
    foreach($requirements as $field => $requirement) {
      if (isset($requirement['dependencies']) && !empty($requirement['dependencies'])) {
        $dependencies = $requirement['dependencies'];

        //only enforce dependencies if the field is required or if its included in the input data.
        if (!$requirement['required'] && !isset($data[$field])) {
          continue;
        }

        if (is_array($dependencies)) {
          foreach($dependencies as $dependency) {
            $this->validateDependency($data, $field, $dependency);
          }
        } else {
          $this->validateDependency($data, $field, $dependencies);
        }
      }
    }
  }

  private function validateDependency(array $data, $field, $dependency) {
    if (!isset($data[$dependency])) {
      throw new InvalidArgumentException('The field \''. $field. '\' has a dependency on the field \'' . $dependency .'\'');
    }

    $requirements[$dependency]['required'] = true;
  }

  private function validateExcludes(array $requirements, array $data) {
    foreach($requirements as $field => $requirement) {
      if (isset($requirement['excludes']) && !empty($requirement['excludes'])) {
        $excludes = $requirement['excludes'];

        //only enforce exclusions if the field is required or if its included in the input data.
        if (!$requirement['required'] && !isset($data[$field])) {
          continue;
        }

        if (is_array($excludes)) {
          foreach($excludes as $exclude) {
            $this->validateExclude($data, $field, $exclude);
          }
        } else {
          $this->validateExclude($data, $field, $excludes);
        }
      }
    }
  }

  private function validateExclude(array $data, $field, $exclude) {
    if (isset($data[$exclude])) {
      throw new InvalidArgumentException('The field \''. $field. '\' prevents you from providing a value for the field \'' . $exclude .'\', both cannot be provided at the same time.');
    }
  }

  private function validateRequirements(array &$requirements) {
    foreach($requirements as $field => $requirement) {
      $requirements[$field] = $this->resolver->resolve($requirement);
    }
  }

  public function validateData(array $requirements, array $data) {
    $parameterBag = new ParameterBag();

    $this->validateRequirements($requirements);

    $this->validateDependencies($requirements, $data);
    $this->validateExcludes($requirements, $data);
    $this->validateFields($parameterBag, $requirements, $data);

    return $parameterBag;
  }

  private function formatData($value, $requirements) {
    $type = isset($requirements['convertToType']) ? $requirements['convertToType'] : '';
    $format = isset($requirements['conversionFormat']) ? $requirements['conversionFormat'] : '';

    switch($type) {
      case self::TYPE_DATETIME:
        return $this->formatDateTime($value, $format);
      case self::TYPE_GENERIC:
        return $this->formatGeneric($value, $format);
      default:
        return $value;
    }
  }

  private function formatDateTime($value, $format) {
    if (!$value instanceof \DateTime) {
      if (empty($format)) {
        return new \DateTime(strtotime($value));
      } else {
        return \DateTime::createFromFormat($format, $value);
      }
    } else {
      return $value;
    }
  }

  private function formatGeneric($value, $format) {
    if (empty($format)) {
      return $value;
    } else {
      if (is_callable($format)) {
        return $format($value);
      } else if (is_array($format)) {
        return call_user_func_array($format, $value);
      } else {
        return $value;
      }
    }
  }
}
