<?php

namespace Tests\Bazookas\CommonBundle\Utility;

use Bazookas\CommonBundle\Utility\Utility;
use Symfony\Bundle\FrameworkBundle\Test\KernelTestCase;

class UtilityTest extends KernelTestCase
{
  private $container;

  public function setUp()
  {
    self::bootKernel();
    $this->container = self::$kernel->getContainer();
  }

  /**
   * Test the 'startsWith' function
   */
  public function testStartsWith() {
    $string = 'This is a test string';

    $this->assertTrue(Utility::startsWith($string, 'This'));

    $this->assertFalse(Utility::startsWith($string, ''));
    $this->assertFalse(Utility::startsWith($string, 'a'));
  }

  /**
   * Test the 'endsWith' function
   */
  public function testEndsWith() {
    $string = 'This is a test string';

    $this->assertFalse(Utility::endsWith($string, 'This'));
    $this->assertFalse(Utility::endsWith($string, ''));

    $this->assertTrue(Utility::endsWith($string, 'g'));
    $this->assertTrue(Utility::endsWith($string, 'string'));
  }

  /**
   * Test the 'generateRandomString' function
   */
  public function testGenerateRandomString() {
    //Assert that the default input returns a 16 char string
    $randomString = Utility::generateRandomString();
    $this->assertTrue(strlen($randomString) === 16);

    //Assert that it only generates a 8 char string using digits
    $alphabet = '0123456789';
    $randomString = Utility::generateRandomString(8, $alphabet);
    $this->assertTrue(strlen($randomString) === 8);
    $this->assertRegExp('/\d+/', $randomString);

    //Assert that it only generates a 128 char string using lower case letters
    $alphabet = 'abcdefghijklmnopqrstuvwxyz';
    $randomString = Utility::generateRandomString(128, $alphabet);
    $this->assertTrue(strlen($randomString) === 128);
    $this->assertRegExp('/[a-z]+/', $randomString);
  }

  /**
   * Test the 'isAssociative' function
   */
  public function testIsAssociative() {
    $input = [ 'test', 'hi', 'something' ];
    $this->assertFalse(Utility::isAssociative($input));

    $input = [ 'test' => 'hi', 'something' => 'something darkside' ];
    $this->assertTrue(Utility::isAssociative($input));

    $input = [ 'test' => 'hi', 'something' ];
    $this->assertTrue(Utility::isAssociative($input));
  }

  /**
   * Test the 'convertTimeToReadable' function
   */
  public function testConvertTimeToReadable() {
    //Convert 360 seconds to 6 minutes
    list($days, $hours, $minutes, $seconds) = Utility::convertTimeToReadable(360);

    $this->assertTrue($days === 0);
    $this->assertTrue($hours === 0);
    $this->assertTrue($minutes === 6);
    $this->assertTrue($seconds === 0);

    //Convert 361 seconds to 6 minutes and 1 second
    list($days, $hours, $minutes, $seconds) = Utility::convertTimeToReadable(361);

    $this->assertTrue($days === 0);
    $this->assertTrue($hours === 0);
    $this->assertTrue($minutes === 6);
    $this->assertTrue($seconds === 1);

    //Convert 13 seconds to 13 seconds
    list($days, $hours, $minutes, $seconds) = Utility::convertTimeToReadable(13);

    $this->assertTrue($days === 0);
    $this->assertTrue($hours === 0);
    $this->assertTrue($minutes === 0);
    $this->assertTrue($seconds === 13);

    //Convert 86401 seconds to 1 day and 1 seconds
    list($days, $hours, $minutes, $seconds) = Utility::convertTimeToReadable(86401);

    $this->assertTrue($days === 1);
    $this->assertTrue($hours === 0);
    $this->assertTrue($minutes === 0);
    $this->assertTrue($seconds === 1);

    //Convert 3610 seconds to 1 hour and 10 seconds
    list($days, $hours, $minutes, $seconds) = Utility::convertTimeToReadable(3610);

    $this->assertTrue($days === 0);
    $this->assertTrue($hours === 1);
    $this->assertTrue($minutes === 0);
    $this->assertTrue($seconds === 10);

    //Assert that negative numbers fail
    $this->assertFalse(Utility::convertTimeToReadable(-20));

    //Assert that null fails
    $this->assertFalse(Utility::convertTimeToReadable(null));
  }

  /**
   * Test the 'getMedianValue' function
   */
  public function testGetMedianValue() {
    //Test odd numbered arrays
    $input = [1, 2, 3, 4, 5];
    $median = Utility::getMedianValue($input);

    $this->assertTrue($median === 3);

    //Test even numbered arrays
    $input = [1, 2, 3, 4];
    $median = Utility::getMedianValue($input);

    $this->assertTrue($median === 2.5);

    //Test an even numbered array of negative numbers
    $input = [-1, -2, -3, -4];
    $median = Utility::getMedianValue($input);

    $this->assertTrue($median === -2.5);
  }
}
