<?php
namespace Bazookas\CronBundle\Command;

use Bazookas\CronBundle\Entity\Cron;
use Bazookas\CronBundle\Entity\CronHistory;
use Bazookas\CronBundle\Exception\CronException;
use Bazookas\CronBundle\Exception\CronExceptionMap;
use Bazookas\CronBundle\Services\Interfaces\CronServiceInterface;
use Doctrine\ORM\EntityManager;
use Symfony\Bundle\FrameworkBundle\Command\ContainerAwareCommand;
use Symfony\Component\Console\Exception\RuntimeException;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Input\InputOption;
use Symfony\Component\Console\Output\OutputInterface;
use Symfony\Component\Filesystem\LockHandler;
use Symfony\Component\Process\Exception\ProcessTimedOutException;
use Symfony\Component\Process\Process;

class RunSingleCronCommand extends ContainerAwareCommand
{
  private $consoleDir;

  public function configure()
  {
    $this
      ->setHidden(true)
      ->setName('crons:run:command')
      ->setDescription('Internal use only, do not run this command manually.')
      ->addOption('cron', 'cron', InputOption::VALUE_REQUIRED, 'The id of the cronjob to execute')
    ;
  }

  public function execute(InputInterface $input, OutputInterface $output)
  {
    $this->consoleDir = 'php '. realpath($this->getContainer()->get('kernel')->getRootDir() . '/../bin/console');

    // get the entity manager and cron instance
    $entityManager = $this->getContainer()->get('doctrine.orm.entity_manager');
    $cron = $this->getCron($entityManager, $input);

    // create a command lock
    $command = $cron->getCommand();
    $commandLock = new LockHandler($command);

    // remember the start date
    $start = new \DateTime("now", new \DateTimeZone('UTC'));

    // validate the cron success service
    $cronService = $this->getService($entityManager, $cron, $start);
    $mailerConfig = $this->getMailerConfig();

    // see if cron is locked and execute it
    if ($commandLock->lock()) {
      $this->executeCommand($entityManager, $commandLock, $cron, $cronService, $start, $mailerConfig);
    }
    else {
      $this->checkLockDuration($entityManager, $commandLock, $cron, $cronService, $start, $mailerConfig);
    }

    $output->writeln('done');
  }

  /**
   * Returns the cron job entity
   * @param EntityManager $entityManager
   * @param InputInterface $input
   * @return Cron
   */
  private function getCron(EntityManager $entityManager, InputInterface $input) {
    $cronRepository = $entityManager->getRepository(Cron::class);
    return $cronRepository->find($input->getOption('cron'));
  }

  /**
   * Gets the service to be executed upon success or failure of the command
   * @param EntityManager $entityManager
   * @param Cron $cron
   * @param $start
   * @return CronServiceInterface
   * @throws CronException
   */
  private function getService(EntityManager $entityManager, Cron $cron, $start) {
    // get the cron service to call upon completion/error of the cronjob
    $cronService = $this->getContainer()->get($cron->getCronService());
    if (!$cronService instanceof CronServiceInterface) {
      self::updateCronHistory(
        $cron,
        $start,
        $cron->getLastRunOn(),
        $start,
        Cron::STATUS_FAILED,
        date('Y-m-d H:i:s') . ' - The cron service must implement the interface: ' . CronServiceInterface::class,
        $cron->getNextRunOn()
      );
      $cron->setStatus(Cron::STATUS_FAILED);

      // persist the cronjob
      $entityManager->persist($cron);
      $entityManager->flush();

      throw new CronException('The cron service must implement the interface: ' . CronServiceInterface::class, CronExceptionMap::INVALID_SERVICE);
    }
    return $cronService;
  }

  /**
   * Gets the mailer configuration for the cron service
   * @return array
   */
  protected function getMailerConfig() {
    $container = $this->getContainer();
    $config = [
      'mailer' => $container->get('swiftmailer.mailer.debug_mailer'),
      'mailer_from' => 'simon@bazookas.be',
      'mailer_to' => 'simon@bazookas.be'
    ];


    if ($container->hasParameter('debug_mailer_from')) {
      $config['mailer_from'] = $container->getParameter('debug_mailer_from');
    }

    if ($container->hasParameter('debug_mailer_to')) {
      $config['mailer_to'] = $container->getParameter('debug_mailer_to');
    }

    /** @var \Symfony\Component\Routing\RequestContext $routerContext */
    $routerContext = $container->get('router')->getContext();
    $config['baseUrl'] = $routerContext->getHost().$routerContext->getBaseUrl();
    $config['kernelRootDir'] = $container->getParameter('kernel.root_dir');

    return $config;
  }

  /**
   * Tries to execute the command itself
   * @param EntityManager $entityManager
   * @param LockHandler $commandLock
   * @param Cron $cron
   * @param CronServiceInterface $cronService
   * @param \DateTime $start
   * @param $mailerConfig
   * @throws \Exception
   */
  private function executeCommand(EntityManager $entityManager, LockHandler $commandLock, Cron $cron, CronServiceInterface $cronService, \DateTime $start, $mailerConfig) {
    // create an output string
    $command = $cron->getCommand();
    $output = $start->format('Y-m-d H:i:s') . ' - Starting execution of ' . $command . "\n\n";

    try {
      $nextRun = new \DateTime("now", new \DateTimeZone('UTC'));
      $nextRun->modify($cron->getExecutionInterval().' '.$cron->getIntervalUnit());

      /*
       * TODO find a better way to handle internal symfony commands?
       * TODO force --env=prod ?
       * for now:
       * replace bin/console with the real path to console
       * the used regex will replace 'php bin/console' or 'bin/console' at the start of the string
       */
      $process = new Process(
        preg_replace('/^(php )?(bin\/console)/', $this->consoleDir, $command),
        null,
        null,
        null,
        $cron->getMaxExecutionTime(),
        [
          'suppress_errors' => false,
        ]
      );


      // actually run the process
      $process->run(function ($type, $buffer) use (&$output) {
        $output .= $buffer;
      });

      // check if the cronjob was successful
      $isSuccessFul = ($process->isSuccessful() && empty($process->getErrorOutput()));

      // set the end date
      $end = new \DateTime("now", new \DateTimeZone('UTC'));
      $output = $output . "\n" . $end->format('Y-m-d H:i:s') . ' - Finished execution of ' . $command;

      //get the status of the command
      $status = $isSuccessFul ? Cron::STATUS_SUCCESS : Cron::STATUS_FAILED;

      //create a cron history item if the command does not remove itself after execution
      if (!$cron->getRemoveAfterExecute()) {
        $this->updateCronHistory($cron, $start, $start, $end, $status, $output, $nextRun);
      }

      // send success or error mails
      if ($isSuccessFul) {
        $cron
          ->setFailureCount(0)
          ->setNotificationSent(false)
        ;
        $cronService->onSuccess($cron, $mailerConfig);
      } else {
        $this->incrementFailCount($cron, $cronService, $mailerConfig);
      }

      // persist or remove the cron job
      if ($cron->getRemoveAfterExecute()) {
        $entityManager->remove($cron);
      }
      else {
        $entityManager->persist($cron);
      }

      $entityManager->flush();
    }
    catch (ProcessTimedOutException $e) {
      $output .= $e->getMessage();
      $this->updateCronHistory($cron, $start, $start, new \DateTime('now', new \DateTimeZone('UTC')), Cron::STATUS_FAILED, $output, $nextRun);
      $this->incrementFailCount($cron, $cronService, $mailerConfig);
      $entityManager->persist($cron);
      $entityManager->flush();
    }
    catch (\Exception $e) {
      throw $e;
    }
    finally {
      $commandLock->release();
    }
  }

  private function incrementFailCount(Cron $cron, CronServiceInterface $cronService, $mailerConfig, $callback = 'onFailure') {
    $cron->incrementFailCount(1);

    if ($cron->getMaxExecutionsBeforeFail() < $cron->getFailureCount()) {
      if (!$cron->isNotificationSent()) {
        //ALARM! send an email!
        $cronService->$callback($cron, $mailerConfig);
        $cron->setNotificationSent(true);
      }

      if (!$cron->isContinueAfterFailCount()) {
        $cron->setStatus(Cron::STATUS_FAILED);
      }
    }
  }

  /**
   * When a lock has been encountered this check will be executed to see if lock duration has exceeded the max execution time. If so, let it try a few times before sending a failure mail.
   * @param EntityManager $entityManager
   * @param LockHandler $commandLock
   * @param Cron $cron
   * @param CronServiceInterface $cronService
   * @param \DateTime $start
   * @param $mailerConfig
   */
  private function checkLockDuration(EntityManager $entityManager, LockHandler $commandLock, Cron $cron, CronServiceInterface $cronService, \DateTime $start, $mailerConfig) {

    // command is still locked...?
    $nextRun = $cron->getNextRunOn();
    $lockDuration = $start->format('U') - $nextRun->format('U');

    if ($lockDuration > $cron->getMaxExecutionTime() + 10) {
      $status = Cron::STATUS_LOCKED;
      $output = date('Y-m-d H:i:s') . ' - Locked since ' . $cron->getNextRunOn()->format('Y-m-d H:i:s') . ' (' . $lockDuration . ' seconds)';
      $lastRun = $cron->getLastRunOn();

      // update the cron history
      $this->updateCronHistory($cron, $start, $lastRun, $start, $status, $output, $nextRun);

      // increment the fail counter
      $cron->incrementFailCount(1);
      $this->incrementFailCount($cron, $cronService, $mailerConfig, 'onLocked');
    }

    // persist everything
    $entityManager->persist($cron);
    $entityManager->flush();
  }

  /**
   * @param Cron $cron
   * @param $start
   * @param $end
   * @param $status
   * @param $output
   * @param $nextRun
   */
  private function updateCronHistory($cron, $start, $lastRunOn, $end, $status, $output, $nextRun) {
    $cronHistory = new CronHistory();
    $cronHistory
      ->setCron($cron)
      ->setRunStart($start)
      ->setRunEnd($end)
      ->setRunStatus($status)
      ->setRunOutput($output)
    ;

    $cron
      ->addHistory($cronHistory)
      ->setLastRunOn($lastRunOn)
      ->setLastRunStatus($status)
      ->setNextRunOn($nextRun)
    ;
  }
}
